@@ -1,11 +1,14 @@
+import asyncio
 import sys
 from dataclasses import dataclass, field
 from typing import Optional, Any, Callable
+import contextlib
 
 from .command import CommandGroup, ShowHelpError, clean_multiline, OnCommandRun
 from .exceptions import (
     CommandNotFoundError, PosParamsCountError,
-    KeywordParamNotFoundError, KeywordParamMissingError
+    KeywordParamNotFoundError, KeywordParamMissingError,
+    InvalidChoiceError
 )
 from .formatter import Formatter, RichFormatter
 
@@ -27,7 +30,6 @@ class Cli:
     For instance, you can use this to get the arguments passed
     for monitoring
     """
-
     _group: CommandGroup = field(init=False, default_factory=CommandGroup)
 
     def __post_init__(self):
@@ -51,8 +53,8 @@ class Cli:
             return self._group.run_with_args(*args)
         except CommandNotFoundError as e:
             e.input_args = args
-            # self.formatter.print_cmd_error(e.valid_commands)
-            raise e
+            self.formatter.print_cmd_error(e.valid_commands)
+            sys.exit(1)
         except ShowHelpError as e:
             self.formatter.print_help(group=e.group,
                                       command=e.command,
@@ -61,32 +63,35 @@ class Cli:
             if not e.cmd:
                 raise NotImplementedError('Got empty command')
             self.formatter.print_keyword_param_error(e.cmd, e.param)
-            return
+            sys.exit(1)
         except KeywordParamMissingError as e:
             if not e.cmd:
                 raise NotImplementedError('Got empty command')
             self.formatter.print_param_error(e.param, e.cmd)
-            return
+            sys.exit(1)
         except PosParamsCountError as e:
             if not e.cmd:
                 raise NotImplementedError('Got empty command')
             self.formatter.print_count_error(e.expected_count, e.count, e.cmd)
-            return
+            sys.exit(1)
+        except InvalidChoiceError as e:
+            self.formatter.print_invalid_value_error(e.value, e.choices)
+            sys.exit(1)
 
-    def command(self, cmd: str = None, help: str = None, description: str = None):
+    def command(self, cmd: Optional[str] = None, help: Optional[str] = None, description: Optional[str] = None):
         return self._group.command(cmd=cmd, help=help, description=description)
 
     def processor(self):
         return self._group.processor()
 
-    def add_option(self, *args, help: str = None, data_type: Any = bool, default: Any = False):
+    def add_option(self, *args, help: Optional[str] = None, data_type: Any = bool, default: Any = False):
         self._group.add_option(*args, help=help, data_type=data_type, default=default)
 
     def set_options_processor(self, fn: Callable):
         """ Function to call with all the options before running `run` or `run_with_args`"""
         self._group.set_options_processor(fn)
 
-    def add_command(self, cmd: str, f, help: str = None, description: str = None):
+    def add_command(self, cmd: str, f, help: Optional[str] = None, description: Optional[str] = None):
         self._group.add_command(cmd=cmd, f=f, help=help, description=description)
 
     def add_command_group(self, group: CommandGroup):
