@@ -2,8 +2,8 @@ import asyncio
 import textwrap
 from dataclasses import dataclass, field
 from functools import wraps
-from inspect import getdoc, iscoroutinefunction
-from typing import get_type_hints, Optional, Any, NamedTuple, Callable
+from inspect import getdoc
+from typing import Optional, Any, NamedTuple, Callable
 
 from .exceptions import (
     DuplicatedCommandError, CommandException, CommandNotFoundError
@@ -11,8 +11,9 @@ from .exceptions import (
 from .utils import (
     CommandOption,
     CommandDerivedOption,
-    get_default_args,
+    extract_function_info,
     parse_input_args,
+    run_function,
     convert_args_to_dict
 )
 
@@ -47,16 +48,20 @@ class Command:
 
     @property
     def keyword_args(self) -> list[CommandOption]:
-        return [opt for opt in self.options if not opt.is_positional_arg]
+        return sorted([opt for opt in self.options if not opt.is_positional_arg],
+                      key=lambda x: (-x.is_required, x.name))
 
-    def run(self, *args, loop: asyncio.AbstractEventLoop = None, **kwargs):
-        if iscoroutinefunction(self.fn):
-            if loop is not None:
-                return loop.run_until_complete(self.fn(*args, **kwargs))
-            else:
-                return asyncio.run(self.fn(*args, **kwargs))
-        else:
-            return self.fn(*args, **kwargs)
+    @property
+    def options_sorted(self) -> list[CommandOption]:
+        """ Sorts with the following order:
+         - positional
+         - keyword required
+         - keyword optional
+        """
+        return self.positional_args + self.keyword_args
+
+    def run(self, *args, **kwargs):
+        run_function(self.fn, *args, **kwargs)
 
     def __post_init__(self):
 
@@ -74,29 +79,6 @@ class Command:
                 _keyword_args.add(_keyword_arg)
 
 
-def extract_function_info(f) -> tuple[list[CommandOption], list[CommandDerivedOption]]:
-    """Extracts the options from a function arguments"""
-    options: list[CommandOption] = []
-    derived_opts: list[CommandDerivedOption] = []
-    defaults: list[CommandOption] = get_default_args(f)
-
-    for (param_name, param_type), option in zip(get_type_hints(f).items(),
-                                                defaults):
-        if isinstance(option, CommandOption):
-            option.name = param_name
-            option.data_type = param_type
-            options.append(option)
-        elif isinstance(option, CommandDerivedOption):
-            option.param_name = param_name
-            _options, _ = extract_function_info(option.processor)
-            options += _options
-            derived_opts.append(option)
-        else:
-            pass
-
-    return options, derived_opts
-
-
 @dataclass
 class CommandMeta:
     cmd_name: str
@@ -126,22 +108,24 @@ class CommandGroup:
     _commands: dict[str, Command] = field(init=False, default_factory=dict)
     _command_groups: dict[str, 'CommandGroup'] = field(init=False, default_factory=dict)
 
+    #
+
     def __post_init__(self):
         self.description = clean_multiline(self.description) if self.description else None
 
     @property
-    def commands(self):
+    def commands(self) -> dict:
         return {k: self._commands.get(k) or self._command_groups[k] for k in sorted(self.command_names)}
 
     @property
     def options(self):
-        return self._options
+        return sorted(self._options, key=lambda x: x.is_required, reverse=True)
 
     def add_sub_parser(self, help: Optional[str] = None, description: Optional[str] = None):
         cls = type(self)
         return cls(help=help, description=description)  # noqa
 
-    def add_option(self, *args: str, help: str = None, data_type: Any = bool,
+    def add_option(self, *args: str, help: Optional[str] = None, data_type: Any = bool,
                    default: Any = False):
         opt = CommandOption(
             default=default,
@@ -165,7 +149,7 @@ class CommandGroup:
         group.on_cmd_run = self.on_cmd_run
         self._command_groups[group.name] = group
 
-    def add_command(self, f, cmd: str = None, help: str = None, description: str = None):
+    def add_command(self, f, cmd: Optional[str] = None, help: Optional[str] = None, description: Optional[str] = None):
         cmd_name = cmd or f.__name__
         if cmd_name in self.commands:
             raise DuplicatedCommandError(f'Duplicated command found for {cmd_name!r}',
@@ -179,7 +163,7 @@ class CommandGroup:
                                            help=help,
                                            description=description or getdoc(f))
 
-    def command(self, cmd: str = None, help: str = None, description: str = None):
+    def command(self, cmd: Optional[str] = None, help: Optional[str] = None, description: Optional[str] = None):
 
         def _command(f):
             @wraps(f)
@@ -205,7 +189,8 @@ class CommandGroup:
 
         return _processor
 
-    def run_with_args(self, *args, parent_args: ParentArgs = None):
+    def run_with_args(self, *args,
+                      parent_args: Optional[ParentArgs] = None):
 
         cmd, global_options, cmd_options = parse_input_args(args, self.command_names)
 
@@ -280,8 +265,8 @@ class CommandGroup:
 class ShowHelpError(Exception):
     def __init__(self,
                  group: CommandGroup,
-                 command: Command = None,
-                 parent_args: ParentArgs = None):
+                 command: Optional[Command] = None,
+                 parent_args: Optional[ParentArgs] = None):
         self.command = command
         self.group = group
         self.parent_args = parent_args
